@@ -1,10 +1,19 @@
-from typing import List, Tuple
+from typing import List, Tuple, Any
 from copy import deepcopy
 from .handlers import Handler
 
 
+class Reject:
+    pass
+
+
+class Reset:
+    pass
+
+
 class BoundChain:
-    def __init__(self, handlers: List[Handler], user_id: int):
+    def __init__(self, primal_chain: Chain, handlers: List[Handler], user_id: int):
+        self.primal_chain = primal_chain
         self.handlers = handlers
         self.user_id = user_id
         self.data = None
@@ -13,15 +22,19 @@ class BoundChain:
         return event_type == self.handlers[0].event_type and user_id == self.user_id
 
     async def new_event(self, token: str, event: dict, event_params: dict) -> Tuple:
-        result = None
-
-        handler = self.handlers.pop(0)
+        handler = self.handlers[0]
         if handler.is_trigger(event_params):
             self.data = await handler.new_event(token, event, event_params, self.data)
-            result = True
+            if isinstance(self.data, Reject):
+                result = False
+            else:
+                result = True
         else:
             result = False
 
+        if result and not isinstance(self.data, Reset):
+            self.handlers.pop(0)
+
         if len(self.handlers) < 1:
             result = False
 
@@ -40,4 +53,4 @@ class Chain:
         return self.handlers[0].is_trigger(event_params)
 
     def bind_chain(self, user_id: int) -> BoundChain:
-        return BoundChain(deepcopy(self.handlers), user_id)
+        return BoundChain(self, deepcopy(self.handlers), user_id)
